td.original p {
  visibility: hidden
}

.inline-group .tabular tr.has_original td {
    padding-top: 10px;
}

.column-forminstance {
    font-size: 0;
}

.small_admin_button {
    padding: 5px 10px !important;
}

#responses_container {
    overflow-x: auto;
}

#responses_table {
    width: 100%;
}

.post_instance {
    vertical-align: middle;
    white-space: nowrap;
    border-right: 1px dotted #656565 !important;
}

.post_instance.no_right_border {
    border-right: 0 !important;
}

.post_instance.tally_cell {
    font-weight: 700;
    line-height: 1.5em;
    padding-bottom: 0;
    border-bottom: 0 !important;
}

.tally_row {
    background-color: transparent !important;
}
