@charset "UTF-8";


#forms_content {
    background-color: #ffffff;
    font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
    word-break: normal;
    overflow: hidden;
    padding: 0 0 2em 0;
}

#forms_content h1 {
    font-size: 1.5em;
    font-weight: 500;
    margin: 0.75em 0 0.75em 0;
}

#forms_content h2 {
    font-size: 1.25em;
    font-weight: 500;
    margin: 1.25em 0 0.75em 0;
}

#forms_content h3 {
    font-size: 1em;
    font-weight: 600;
    margin: 1.25em 0 -0.75em 0;
}

#forms_content p {
    font-size: 1em;
    line-height: 1.4em;
}

#forms_content ul {
    margin: 0 0 0 0;
    line-height: 1.4em;
}

#forms_content ul li {
    list-style: disc;
    margin: 0 0 0.25em 0;
}

#forms_content ul li ul {
    margin: 1.2em 0 0 0;
}

#forms_content ul li ul li {
    list-style: circle;
    margin: 0 0 0.5em 0;
}

#forms_content ol {
    margin: 0 0 0 0;
    line-height: 1.4em;
}

#forms_content ol li {
    margin: 0 0 0.25em 0;
}

#forms_content ol li ol {
    margin: 1.2em 0 0 0;
}

#forms_content ol li ol li {
    margin: 0 0 0.5em 0;
}

#forms_content > a, a:link, a:visited, a:active, a:hover {
    color:#656565;
    text-decoration: underline;
}

#forms_content form {
    max-width: 500px;
}

#forms_content label {
    display: block;
    box-sizing: border-box;
    width: 100%;
}

#forms_content input[type=text], input[type=email], input[type=checkbox], select, textarea {
    margin-top: 0.2em;
    margin-left: 0;
    box-sizing: border-box;
    font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
    font-size: 1em;
    border-color: #656565;
}

#forms_content input[type=text], input[type=email], select, textarea {
    display: block;
    border-width: 1px;
}

#forms_content input[type=text], input[type=email], textarea {
    width: 100%;
}

#forms_content select {
}

#forms_content textarea {
    resize: vertical;
    height: 200px;
    min-height: 200px;
}

#forms_content input[type='submit'] {
	font-weight: 500;
	padding: 0.5em 2em;
	color: #656565;
	background-color: #ededed;
	border: 0;
	border-radius: 0.2em;
	cursor: pointer;
    -webkit-appearance: none;
}

.checkbox_select_multiple {
  margin: 0;
  padding: 0;
}

.checkbox_select_multiple li {
  list-style-type: none;
  padding: 0 0 0.5em 1em;
}

.forms_error {
    display: block;
    font-size: 0.75em;
    color: #FF0000;
    margin: -0.5em 0 0.5em 0;
}

/* Begin hidden field */
#id_email {
    display: none;
}
/* End hidden field */

.g-recaptcha {
    padding-top: 1em;
}