{% extends 'base.html' %}


{% block title %}
    {{ title }}
{% endblock %}

{% block forms_head %}
    <link rel="stylesheet" type="text/css" href="{{ css_style }}">
{% endblock %}


{% block content %}

    <div id="forms_content">

        <h1>
            {{ title }}
        </h1>

        <p>
            Thank you, the form has been submitted.
        </p>

    </div>

{% endblock %}
