{% extends 'django_simple_forms/internal/internal_base.html' %}

{% block content %}

    <table>

       {% for key, value in output_formatted_date_time.items %}
            <tr>
                <td class="td_left">
                    {{ value.0 }}
                </td>
                <td class="td_right">
                    {% for item in value %}
                        {% if forloop.counter0 > 0 %}
                            {% if forloop.counter0 > 1 %}
                                <br><br>
                            {% endif %}
                            {% if item is True %}
                                &#10003;
                            {% elif item is False %}
                                &#10005;
                            {% else %}
                                {{ item|safe }}
                            {% endif %}
                        {% endif %}
                    {% endfor %}
                </td>
            </tr>
        {% endfor %}

        <tr>
            <td class="td_left">
                Created:
            </td>
            <td class="td_right">
                {{ created }}
            </td>
        </tr>

        <tr>
            <td class="td_left">
                Updated:
            </td>
            <td class="td_right">
                {{ updated }}
            </td>
        </tr>

    </table>

    <form method="post">

        {% csrf_token %}

        <p>
            <input type="submit" name="return" value="Return">
        </p>
        <br>
       {% if delete_perm %}
            {%  if delete_post %}
                <p id="warning">
                    Are you certain you want to delete this response (this is not reversible)?
                </p>
                <br>
                <p>
                    <input type="submit" name="confirm" value="Confirm delete">
                </p>
            {%  else %}
            <p>
                <input type="submit" name="delete" value="Delete">
            </p>
            {%  endif %}
        {%  endif %}

    </form>

{% endblock %}
