@charset "UTF-8";


body {
    background-color: #ffffff;
    color: #000000;
    font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
    padding: 0.5em 1em 1em 1em;
}

h1 {
    font-size: 1.25em;
    font-weight: 500;
    margin: 0.75em 0 0.75em 0;
}

p {
    font-size: 1em;
    line-height: 1.4em;
}

form {
    max-width: 500px;
}

label {
    display: block;
    box-sizing: border-box;
    width: 100%;
    color: #656565;
}

input[type=text], input[type=email], input[type=checkbox], select, textarea {
    margin-top: 0.2em;
    margin-left: 0;
    box-sizing: border-box;
    font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
    font-size: 1em;
    border-color: #656565;
}

input[type=text], input[type=email], select, textarea {
    display: block;
    border-width: 1px;
}

input[type=text], input[type=email], textarea {
    width: 100%;
}

input[type='submit'] {
	font-weight: 500;
	padding: 0.5em 2em;
	color: #656565;
	background-color: #ededed;
	border: 0;
	border-radius: 0.2em;
	cursor: pointer;
    -webkit-appearance: none;
}

select {
    border-color: #656565;
}

textarea {
    resize: vertical;
    height: 200px;
    min-height: 200px;
}

.checkbox_select_multiple {
    margin: 0;
    padding: 0;
}

.checkbox_select_multiple li {
    list-style-type: none;
    padding: 0 0 0.5em 1em;
}

.forms_error {
    display: block;
    font-size: 0.75em;
    color: #FF0000;
    margin: -0.5em 0 0.5em 0;
}

table {
    border-collapse: collapse;
    margin: 1em 0 3em 0;
}

td {
    vertical-align: top;
    border-bottom: 1px solid #656565;
    padding: 1em 1em 1em 0;
}

.td_left {
    color: #656565;
    padding-right: 1.5em;
}

.td_right {
    color: #000000;
}

#warning {
    color: #FF0000;
}
