{% load static %}

<!DOCTYPE html>

<html lang="en">

    <head>

        <title>
            {{ title }}
        </title>

        <link rel="stylesheet" type="text/css" href="{% static "django_simple_forms/internal/css/post_html.css" %}">

        {% block forms_head %}{% endblock %}

    </head>

    <body>

        <h1>
            {{ title }}
        </h1>

        {% block content %}{% endblock %}

        {% block forms_footer %}{% endblock %}

    </body>

</html>
